require 'active_model'
require 'validates_cpf/cpf'

module ActiveModel
  module Validations
    class CpfValidator < ActiveModel::EachValidator
      include ValidatesCpf

      def validate_each(record, attribute, value)
        return if (options[:allow_nil] and value.nil?) or (options[:allow_blank] and value.blank?)
        return if (options[:if] == false) or (options[:unless] == true)
        return if (options[:on].to_s == 'create' and not record.new_record?) or (options[:on].to_s == 'update' and record.new_record?)

        if value.to_s.gsub(/[^0-9]/, '').length <= 11
          if (not value.to_s.match(/\A\d{11}\z/) and not value.to_s.match(/\A\d{3}\.\d{3}\.\d{3}\-\d{2}\z/)) or not Cpf.valid?(value)
            record.errors.add(attribute, 'The CPF is invalid')
          end
        end
      end
    end

    module HelperMethods
      def validates_cpf(*attr_names)
        raise ArgumentError, "You need to supply at least one attribute" if attr_names.empty?
        validates_with CpfValidator, _merge_attributes(attr_names)
      end
    end
  end
end
